#!/usr/bin/env python

# Copyright Contributors to the OpenImageIO project.
# SPDX-License-Identifier: Apache-2.0
# https://github.com/AcademySoftwareFoundation/OpenImageIO

from __future__ import print_function
from __future__ import absolute_import
import math, os
import OpenImageIO as oiio
from OpenImageIO import ImageBuf, ImageSpec, ImageBufAlgo, ROI


OIIO_TESTSUITE_ROOT = os.getenv('OIIO_TESTSUITE_ROOT', '')
OIIO_TESTSUITE_IMAGEDIR = os.getenv('OIIO_TESTSUITE_IMAGEDIR', '')

def make_constimage (xres, yres, chans=3, format=oiio.UINT8, value=(0,0,0),
                xoffset=0, yoffset=0) :
    spec = ImageSpec (xres,yres,chans,format)
    spec.x = xoffset
    spec.y = yoffset
    b = ImageBuf (spec)
    oiio.ImageBufAlgo.fill (b, value)
    return b


def write (image, filename, format=oiio.UNKNOWN) :
    if not image.has_error :
        image.write (filename, format)
    if image.has_error :
        print ("Error writing", filename, ":", image.geterror())

def dumpimg (image, fmt="{:.3f}", msg="") :
    spec = image.spec()
    print (msg, end="")
    if image.has_error :
        print ("Error({})".format(image.geterror()))
    else :
        for y in range(spec.y, spec.y+spec.height) :
            for x in range(spec.x, spec.x+spec.width) :
                p = image.getpixel (x, y)
                print ("[", end="")
                for c in range(spec.nchannels) :
                    print (fmt.format(p[c]), end=" ")
                print ("] ", end="")
            print ("")


# Test an ImageBufAlgo function `func`, with a given set of arguments, running
# both the variety that returns an ImageBuf with the result, and also the kind
# that modifies an existing ImageBuf in place. An error is printed if the
# results differ. The "returned" IB is returned from the function.
def test_iba (func, *args, **kwargs) :
    # Test the version of func that returns an IB
    # func = getattr(ImageBufAlgo, funcname)
    b = func(*args, **kwargs)
    # if filename is not None :
    #     write (b, filename)
    # Test the version of func that writes to an existing IB and make
    # sure it matches.
    b2 = ImageBuf()
    func (b2, *args, **kwargs)
    compresults = ImageBufAlgo.compare (b, b2, 0.0, 0.0)
    if compresults.nfail > 0 :
        print ("FAILURE:", func.__name__, ": IB-returning and in-place versions differed")
        b.write (func.__name__ + "-1.tif")
        b2.write (func.__name__ + "-2.tif")
        exit (1)
    return b


######################################################################
# main test starts here

try:
    # Some handy images to work with
    gridname = os.path.join(OIIO_TESTSUITE_IMAGEDIR, "grid.tif")
    grid = ImageBuf (gridname)
    checker = ImageBuf(ImageSpec(256, 256, 3, oiio.UINT8))
    ImageBufAlgo.checker (checker, 8, 8, 8, (0,0,0), (1,1,1))
    gray128 = make_constimage (128, 128, 3, oiio.HALF, (0.5,0.5,0.5))
    gray64 = make_constimage (64, 64, 3, oiio.HALF, (0.5,0.5,0.5))
    tahoetiny = ImageBuf(OIIO_TESTSUITE_ROOT+"/common/tahoe-tiny.tif")

    # black
    # b = ImageBuf (ImageSpec(320,240,3,oiio.UINT8))
    # b = ImageBufAlgo.zero (roi=oiio.ROI(0,320,0,240,0,1,0,3))
    # b = test_iba ("black.tif", "zero", roi=oiio.ROI(0,320,0,240,0,1,0,3))
    b = test_iba (ImageBufAlgo.zero, roi=oiio.ROI(0,320,0,240,0,1,0,3))
    write (b, "black.tif", oiio.UINT8)

    # fill (including use of ROI)
    b = ImageBuf (ImageSpec(256,256,3,oiio.UINT8));
    ImageBufAlgo.fill (b, (1,0.5,0.5))
    ImageBufAlgo.fill (b, (0,1,0), oiio.ROI(100,180,100,180))
    write (b, "filled.tif", oiio.UINT8)

    # checker
    b = ImageBuf (ImageSpec(256,256,3,oiio.UINT8))
    ImageBufAlgo.checker (b, 64, 64, 64, (1,.5,.5), (.5,1,.5), 10, 5)
    write (b, "checker.tif", oiio.UINT8)

    # noise-uniform
    b = ImageBufAlgo.noise ("white", 0.25, 0.75, roi=ROI(0,64,0,64,0,1,0,3))
    write (b, "noise-uniform3.tif", oiio.UINT8)

    # noise-blue
    b = ImageBufAlgo.noise ("blue", 0.25, 0.75, roi=ROI(0,64,0,64,0,1,0,3))
    write (b, "noise-blue3.tif", oiio.UINT8)

    # noise-gaussian
    b = ImageBufAlgo.noise ("gaussian", 0.5, 0.1, roi=ROI(0,64,0,64,0,1,0,3))
    write (b, "noise-gauss.tif", oiio.UINT8)

    # noise-salt
    b = ImageBufAlgo.noise ("salt", 1, 0.01, roi=ROI(0,64,0,64,0,1,0,3))
    write (b, "noise-salt.tif", oiio.UINT8)

    # bluenoise_image
    b = ImageBufAlgo.bluenoise_image()
    b = ImageBufAlgo.crop(b, ROI(0,64,0,64,0,1,0,3))
    write (b, "bluenoise_image3.tif", oiio.UINT8)

    # channels, channel_append
    b = ImageBufAlgo.channels (grid, (0.25,2,"G"))
    write (b, "chanshuffle.tif")
    b = ImageBufAlgo.channels (ImageBuf(OIIO_TESTSUITE_ROOT+"/oiiotool-copy/src/rgbaz.exr"),
                               ("R","G","B","A"))
    write (b, "ch-rgba.exr")
    b = ImageBufAlgo.channels (ImageBuf(OIIO_TESTSUITE_ROOT+"/oiiotool-copy/src/rgbaz.exr"), ("Z",))
    write (b, "ch-z.exr")
    b = test_iba (ImageBufAlgo.channel_append, ImageBuf("ch-rgba.exr"),
                  ImageBuf("ch-z.exr"))
    write (b, "chappend-rgbaz.exr")

    # flatten
    b = test_iba (ImageBufAlgo.flatten, ImageBuf(OIIO_TESTSUITE_ROOT+"/oiiotool-deep/src/deepalpha.exr"))
    write (b, "flat.exr")

    # deepen
    b = test_iba (ImageBufAlgo.deepen, ImageBuf(OIIO_TESTSUITE_ROOT+"/oiiotool-deep/src/az.exr"))
    write (b, "deepen.exr")

    # crop
    b = test_iba (ImageBufAlgo.crop, grid, oiio.ROI(50,150,200,600))
    write (b, "crop.tif")

    # cut
    b = test_iba (ImageBufAlgo.cut, grid, oiio.ROI(50,150,200,600))
    write (b, "cut.tif")

    # paste
    b = ImageBuf()
    b.copy (checker)
    ImageBufAlgo.paste (b, 150, 75, 0, 0, grid)
    write (b, "pasted.tif")

    # rotate90
    b = test_iba (ImageBufAlgo.rotate90, ImageBuf(OIIO_TESTSUITE_ROOT+"/oiiotool/src/image.tif"))
    write (b, "rotate90.tif")

    # rotate180
    b = test_iba (ImageBufAlgo.rotate180, ImageBuf(OIIO_TESTSUITE_ROOT+"/oiiotool/src/image.tif"))
    write (b, "rotate180.tif")

    # rotate270
    b = test_iba (ImageBufAlgo.rotate270, ImageBuf(OIIO_TESTSUITE_ROOT+"/oiiotool/src/image.tif"))
    write (b, "rotate270.tif")

    # flip
    b = test_iba (ImageBufAlgo.flip, ImageBuf(OIIO_TESTSUITE_ROOT+"/oiiotool/src/image.tif"))
    write (b, "flip.tif")

    # flop
    b = test_iba (ImageBufAlgo.flop, ImageBuf(OIIO_TESTSUITE_ROOT+"/oiiotool/src/image.tif"))
    write (b, "flop.tif")

    # reorient
    image_small = ImageBuf()
    ImageBufAlgo.resample (image_small, ImageBuf(OIIO_TESTSUITE_ROOT+"/oiiotool/src/image.tif"),  roi=oiio.ROI(0,160,0,120))
    image_small = ImageBufAlgo.rotate90 (image_small)
    image_small.specmod().attribute ("Orientation", 8)
    b = test_iba (ImageBufAlgo.reorient, image_small)
    write (b, "reorient1.tif")
    image_small = ImageBuf()

    # transpose
    b = test_iba (ImageBufAlgo.transpose, ImageBuf(OIIO_TESTSUITE_ROOT+"/oiiotool/src/image.tif"))
    write (b, "transpose.tif")

    # circular_shift
    b = test_iba (ImageBufAlgo.circular_shift, ImageBuf(OIIO_TESTSUITE_ROOT+"/oiiotool/src/image.tif"), 100, 50)
    write (b, "cshift.tif")

    # clamp
    b = ImageBufAlgo.resize (grid, roi=oiio.ROI(0,500,0,500))
    b = ImageBufAlgo.clamp (b, (0.2,0.2,0.2,0.2), (100,100,0.5,1))
    write (b, "grid-clamped.tif", oiio.UINT8)

    b = oiio.ImageBufAlgo.fill((0, 0, 0), (1, 1, 1), roi=oiio.ROI(0,64,0,64,0,1,0,3))
    b = oiio.ImageBufAlgo.clamp(b, 0.25, 0.75)
    write (b, "clamped-with-float.exr", "half")

    # add
    b = test_iba (ImageBufAlgo.add, gray128, 0.25)
    write (b, "cadd1.exr")
    b = ImageBufAlgo.add (gray128, (0, 0.25, -0.25))
    write (b, "cadd2.exr")
    b = test_iba(ImageBufAlgo.add, make_constimage(64, 64, 3, oiio.HALF, (.1, .2, .3)),
                          make_constimage(64,64,3,oiio.HALF,(.1,.1,.1),20,20))
    write (b, "add.exr")

    # sub
    b = test_iba (ImageBufAlgo.sub, make_constimage(64,64,3,oiio.HALF,(.1,.2,.3)),
                          make_constimage(64,64,3,oiio.HALF,(.1,.1,.1),20,20))
    write (b, "sub.exr")
    b = test_iba (ImageBufAlgo.sub, gray128, (0.125, 0.5, 0.25))
    write (b, "csub2.exr")

    # Test --absdiff and --abs
    # First, make a test image that's 0.5 on the left, -0.5 on the right
    a = ImageBuf (ImageSpec(128,128,3,oiio.HALF))
    ImageBufAlgo.fill (a, (0.5,0.5,0.5))
    ImageBufAlgo.fill (a, (-0.25,-0.25,-0.25), oiio.ROI(0,64,0,128))
    b = test_iba (ImageBufAlgo.abs, a)
    write (b, "abs.exr", oiio.HALF)
    b = test_iba (ImageBufAlgo.absdiff, a, (0.2,0.2,0.2))
    write (b, "absdiff.exr", oiio.HALF)
    a = ImageBuf()

    # mul
    b = ImageBufAlgo.mul (gray128, 1.5)
    write (b, "cmul1.exr")
    b = ImageBufAlgo.mul (gray128, (1.5,1,0.5))
    write (b, "cmul2.exr")
    b = test_iba (ImageBufAlgo.mul, make_constimage(64,64,3,oiio.HALF,(.5,.5,.5)),
                          make_constimage(64,64,3,oiio.HALF,(1.5,1,0.5)))
    write (b, "mul.exr", oiio.HALF)

    # mad
    b = test_iba (ImageBufAlgo.mad, make_constimage(64,64,3,oiio.HALF,(.5,.5,.5)),
                          make_constimage(64,64,3,oiio.HALF,(1.5,1,0.5)),
                          make_constimage(64,64,3,oiio.HALF,(0.1,0.1,0.1)))
    write (b, "mad.exr", oiio.HALF)
    b = test_iba (ImageBufAlgo.mad, make_constimage(64,64,3,oiio.HALF,(.5,.5,.5)),
                          (1.5,1,0.5),
                          (0.1,0.1,0.1))
    write (b, "mad2.exr", oiio.HALF)
    b = test_iba (ImageBufAlgo.mad, make_constimage(64,64,3,oiio.HALF,(.5,.5,.5)),
                          (1.5,1,0.5),
                          make_constimage(64,64,3,oiio.HALF,(0.1,0.1,0.1)))
    write (b, "mad3.exr", oiio.HALF)

    # div
    b = test_iba (ImageBufAlgo.div, gray64, make_constimage (64, 64, 3, oiio.HALF, (2.0,1,0.5)))
    write (b, "div.exr", oiio.HALF)
    b = ImageBufAlgo.div (gray64, 2.0)
    write (b, "divc1.exr", oiio.HALF)
    b = ImageBufAlgo.div (gray64, (2.0,1,0.5))
    write (b, "divc2.exr", oiio.HALF)

    # invert
    a = ImageBuf (OIIO_TESTSUITE_ROOT+"/common/tahoe-small.tif")
    b = test_iba (ImageBufAlgo.invert, a)
    write (b, "invert.tif", oiio.UINT8)

    # normalize
    # Construct an image with several values to test normalization
    norm = ImageBuf(ImageSpec(3, 3, 3, "half"))
    ImageBufAlgo.render_point(norm, 0, 0, (0.0, 0.0, 0.0))    # zero
    ImageBufAlgo.render_point(norm, 1, 0, (0.6, 0.0, 0.0))    # x
    ImageBufAlgo.render_point(norm, 2, 0, (-0.6, 0.0, 0.0))   # -x
    ImageBufAlgo.render_point(norm, 0, 1, (0.0, 0.6, 0.0))    # y
    ImageBufAlgo.render_point(norm, 1, 1, (0.0, -0.6, 0.0))   # -y
    ImageBufAlgo.render_point(norm, 2, 1, (0.0, 0.0, 0.6))    # z
    ImageBufAlgo.render_point(norm, 0, 2, (0.0, 0.0, -0.6))   # -z
    ImageBufAlgo.render_point(norm, 1, 2, (0.6, 0.6, -0.6))   # diag3
    ImageBufAlgo.render_point(norm, 2, 2, (0.0, -0.6, -0.6))  # diag2
    write (norm, "norm.exr", "half")
    # and another copy that is offset to .5 center and scaled by 0.5
    normoffset = ImageBufAlgo.add(ImageBufAlgo.mul(norm, 0.5), 0.5)
    write (normoffset, "normoffset.exr", "half")
    # Test various options
    b = test_iba (ImageBufAlgo.normalize, norm)
    write (b, "normalize.exr", "half")
    b = test_iba (ImageBufAlgo.normalize, norm, scale=0.5)
    write (b, "normalize_scale.exr", "half")
    b = test_iba (ImageBufAlgo.normalize, normoffset, inCenter=0.5)
    write (b, "normalize_offsetin.exr", "half")
    b = test_iba (ImageBufAlgo.normalize, norm, outCenter=0.5, scale=0.5)
    write (b, "normalize_offsetscaleout.exr", "half")
    b = test_iba (ImageBufAlgo.normalize, normoffset, inCenter=0.5, outCenter=0.5, scale=0.5)
    write (b, "normalize_offsetscale.exr", "half")

    # pow
    b = ImageBufAlgo.pow (gray128, 2)
    write (b, "cpow1.exr")
    b = ImageBufAlgo.pow (gray128, (2,2,1))
    write (b, "cpow2.exr")

    # channel_sum
    b = test_iba (ImageBufAlgo.channel_sum,
                  ImageBuf(OIIO_TESTSUITE_ROOT+"/common/tahoe-small.tif"),
                  (.2126,.7152,.0722))
    write (b, "chsum.tif", oiio.UINT8)

    # min, max (per element)
    lrramp64 = ImageBufAlgo.fill((0,0,0), (1,1,1), (0,0,0), (1,1,1),
                                 oiio.ROI(0, 64, 0, 64, 0, 1, 0, 3))
    b = test_iba (ImageBufAlgo.max, lrramp64, gray64)
    write (b, "maximg.tif", oiio.UINT8)
    b = test_iba (ImageBufAlgo.max, lrramp64, (0.25, 0.25, 0.25))
    write (b, "maxval.tif", oiio.UINT8)
    b = test_iba (ImageBufAlgo.min, lrramp64, gray64)
    write (b, "minimg.tif", oiio.UINT8)
    b = test_iba (ImageBufAlgo.min, lrramp64, (0.25, 0.25, 0.25))
    write (b, "minval.tif", oiio.UINT8)

    # minchan, maxchan
    b = test_iba (ImageBufAlgo.maxchan, ImageBufAlgo.fill((0,0,0.2), (1,0,0.2), (0,1,0.2), (1,1,0.2),
                                                ROI(0,100,0,100,0,1,0,3)))
    write (b, "maxchan.tif", oiio.UINT8)
    b = test_iba (ImageBufAlgo.minchan, ImageBufAlgo.fill((0,0,0.8), (1,0,0.8), (0,1,0.8), (1,1,0.8),
                                                ROI(0,100,0,100,0,1,0,3)))
    write (b, "minchan.tif", oiio.UINT8)

    # color_map
    b = ImageBufAlgo.color_map (tahoetiny, -1, "inferno")
    write (b, "colormap-inferno.tif", oiio.UINT8)
    b = ImageBufAlgo.color_map (tahoetiny, -1, 3, 3, (.25,.25,.25,0,.5,0,1,0,0))
    write (b, "colormap-custom.tif", oiio.UINT8)

    # premult/unpremult
    b = make_constimage(100,100,4,oiio.FLOAT,(.1,.1,.1,1))
    ImageBufAlgo.fill (b, (.2,.2,.2,.5), oiio.ROI(50,80,50,80))
    b = test_iba (ImageBufAlgo.unpremult, b)
    write (b, "unpremult.tif")
    b = test_iba (ImageBufAlgo.premult, b)
    write (b, "premult.tif")

    b = ImageBufAlgo.contrast_remap (tahoetiny, black=0.1, white=0.75)
    write (b, "contrast-stretch.tif")
    b = ImageBufAlgo.contrast_remap (tahoetiny, min=0.1, max=0.75)
    write (b, "contrast-shrink.tif")
    b = ImageBufAlgo.contrast_remap (tahoetiny, scontrast=5.0)
    write (b, "contrast-sigmoid5.tif")

    b = ImageBufAlgo.saturate (tahoetiny, scale = 0.0)
    write (b, "saturate-0.tif")
    b = test_iba (ImageBufAlgo.saturate, tahoetiny, scale = 2.0)
    write (b, "saturate-2.tif")

    b = ImageBuf (OIIO_TESTSUITE_ROOT+"/common/tahoe-small.tif")
    b = test_iba (ImageBufAlgo.rangecompress, b)
    write (b, "rangecompress.tif", oiio.UINT8)
    b = test_iba (ImageBufAlgo.rangeexpand, b)
    write (b, "rangeexpand.tif", oiio.UINT8)

    # FIXME - ociolook, ociodisplay, ociofiletransform need tests
    print ("\nTesting color conversions:")
    b = make_constimage (2,2,4,oiio.FLOAT,(0,0,0,1))
    b.setpixel(1, 0, (.25,.25,.25,1))
    b.setpixel(0, 1, (.5,.5,.5,1))
    b.setpixel(1, 1, (1,1,1,1))
    dumpimg (b, msg="linear src=")
    r = test_iba (ImageBufAlgo.colorconvert, b, "Linear", "sRGB")
    dumpimg (r, msg="to srgb =")
    r = ImageBufAlgo.colorconvert(r, "sRGB", "Linear")
    dumpimg (r, msg="back to linear =")
    # Just to test, make a matrix that halves red, doubles green,
    # adds 0.1 to blue.
    M = ( 0.5, 0, 0,   0,
          0,   2, 0,   0,
          0,   0, 1,   0,
          0,   0, 0.1, 1)
    r = ImageBufAlgo.colormatrixtransform (b, M)
    dumpimg (r, msg="after *M =")

    # computePixelStats
    b = ImageBuf (OIIO_TESTSUITE_ROOT+"/common/tahoe-small.tif")
    stats = ImageBufAlgo.computePixelStats (b)
    print ("Stats for tahoe-small.tif:")
    print ("  min         = ", stats.min)
    print ("  max         = ", stats.max)
    print ("  avg         = ", stats.avg)
    print ("  stddev      = ", stats.stddev)
    print ("  nancount    = ", stats.nancount)
    print ("  infcount    = ", stats.infcount)
    print ("  finitecount = ", stats.finitecount)

    # Absolute compare
    compresults = ImageBufAlgo.compare (ImageBuf("flip.tif"), ImageBuf("flop.tif"),
                                        1.0e-6, 1.0e-6)
    print ("Comparison: of flip.tif and flop.tif")
    print ("  mean = %.5g" % compresults.meanerror)
    print ("  rms  = %.5g" % compresults.rms_error)
    print ("  PSNR = %.5g" % compresults.PSNR)
    print ("  max  = %.5g" % compresults.maxerror)
    print ("  max @", (compresults.maxx, compresults.maxy, compresults.maxz, compresults.maxc))
    print ("  warns", compresults.nwarn, "fails", compresults.nfail)

    # Relative compare
    compresults = ImageBufAlgo.compare (ImageBuf("flip.tif"), ImageBuf("flop.tif"),
                                        0.0, 0.0, 0.1, 0.05)
    print ("Relative comparison: of flip.tif and flop.tif")
    print ("  warns", compresults.nwarn, "fails", compresults.nfail)

    # compare_Yee,
    # isConstantColor, isConstantChannel

    b = ImageBuf (ImageSpec(256,256,3,oiio.UINT8));
    ImageBufAlgo.fill (b, (1,0.5,0.5))
    r = ImageBufAlgo.isConstantColor (b)
    print ("isConstantColor on pink image is (%.5g %.5g %.5g)" % r)
    r = ImageBufAlgo.isConstantColor (checker)
    print ("isConstantColor on checker is ", r)

    b = ImageBuf("cmul1.exr")
    print ("Is", b.name, "monochrome? ", ImageBufAlgo.isMonochrome(b))
    b = ImageBuf("cmul2.exr")
    print ("Is", b.name, "monochrome? ", ImageBufAlgo.isMonochrome(b))


    # color_count

    b = ImageBufAlgo.fill (top=(0,0,0), bottom=(1,1,1), roi=ROI(0,4,0,4,0,1,0,3))
    counts = ImageBufAlgo.color_range_check (b, low=0.25, high=(0.5,0.5,0.5))
    print ('color range counts = ', counts)

    # nonzero_region
    b = make_constimage (256,256,3,oiio.UINT8,(0,0,0))
    ImageBufAlgo.fill (b, (0,0,0))
    ImageBufAlgo.fill (b, (0,1,0), oiio.ROI(100,180,100,180))
    print ("Nonzero region is: ", ImageBufAlgo.nonzero_region(b))

    # resize
    b = test_iba (ImageBufAlgo.resize, grid, roi=oiio.ROI(0,256,0,256))
    write (b, "resize.tif")

    # resample
    b = test_iba (ImageBufAlgo.resample, grid, roi=oiio.ROI(0,128,0,128))
    write (b, "resample.tif")

    # fit
    b = test_iba (ImageBufAlgo.fit, grid, roi=oiio.ROI(0,360,0,240))
    write (b, "fit.tif")

    # warp
    Mwarp = (0.7071068, 0.7071068, 0, -0.7071068, 0.7071068, 0, 128, -53.01933, 1)
    b = test_iba (ImageBufAlgo.warp, ImageBuf("resize.tif"), Mwarp)
    write (b, "warped.tif")

    # rotate
    b = test_iba (ImageBufAlgo.rotate, ImageBuf("resize.tif"), math.radians(45.0))
    write (b, "rotated.tif")
    b = ImageBufAlgo.rotate (ImageBuf("resize.tif"), math.radians(45.0), 50.0, 50.0)
    write (b, "rotated-offcenter.tif")

    # make_kernel
    bsplinekernel = ImageBufAlgo.make_kernel ("bspline", 15, 15)
    write (bsplinekernel, "bsplinekernel.exr")

    # convolve -- test with bspline blur
    b = test_iba (ImageBufAlgo.convolve, ImageBuf(OIIO_TESTSUITE_ROOT+"/common/tahoe-small.tif"),
                               bsplinekernel)
    write (b, "bspline-blur.tif", oiio.UINT8)

    # median filter
    b = test_iba (ImageBufAlgo.median_filter, ImageBuf(OIIO_TESTSUITE_ROOT+"/common/tahoe-small.tif"), 5, 5)
    write (b, "tahoe-median.tif", oiio.UINT8)

    # Dilate/erode
    undilated = ImageBuf(OIIO_TESTSUITE_ROOT+"/oiiotool/src/morphsource.tif")
    b = test_iba (ImageBufAlgo.dilate, undilated, 3, 3)
    write (b, "dilate.tif", oiio.UINT8)
    b = test_iba (ImageBufAlgo.erode, undilated, 3, 3)
    write (b, "erode.tif", oiio.UINT8)
    undilated = None

    # unsharp_mask
    b = test_iba (ImageBufAlgo.unsharp_mask, ImageBuf(OIIO_TESTSUITE_ROOT+"/common/tahoe-small.tif"),
                                   "gaussian", 3.0, 1.0, 0.0)
    write (b, "unsharp.tif", oiio.UINT8)

    # unsharp_mark with median filter
    b = ImageBufAlgo.unsharp_mask (ImageBuf(OIIO_TESTSUITE_ROOT+"/common/tahoe-small.tif"),
                                   "median", 3.0, 1.0, 0.0)
    write (b, "unsharp-median.tif", oiio.UINT8)

    # laplacian
    b = test_iba (ImageBufAlgo.laplacian, ImageBuf(OIIO_TESTSUITE_ROOT+"/common/tahoe-tiny.tif"))
    write (b, "tahoe-laplacian.tif", oiio.UINT8)

    # computePixelHashSHA1
    print ("SHA-1 of bsplinekernel.exr is: " +
           ImageBufAlgo.computePixelHashSHA1(bsplinekernel))

    # fft, ifft
    blue = ImageBufAlgo.channels (ImageBuf(OIIO_TESTSUITE_ROOT+"/common/tahoe-tiny.tif"), (2,))
    fft = test_iba (ImageBufAlgo.fft, blue)
    write (fft, "fft.exr", oiio.FLOAT)
    inv = test_iba (ImageBufAlgo.ifft, fft)
    b = ImageBufAlgo.channels (inv, (0,))
    write (b, "ifft.exr", oiio.FLOAT)
    inv.clear()
    fft.clear()

    fft = ImageBuf("fft.exr")
    polar = test_iba (ImageBufAlgo.complex_to_polar, fft)
    b = test_iba (ImageBufAlgo.polar_to_complex, polar)
    write (polar, "polar.exr", oiio.FLOAT)
    write (b, "unpolar.exr", oiio.FLOAT)
    fft.clear()
    polar.clear()

    # fixNonFinite
    bad = ImageBuf (OIIO_TESTSUITE_ROOT+"/oiiotool-fixnan/src/bad.exr")
    b = ImageBufAlgo.fixNonFinite (bad, oiio.NONFINITE_BOX3)
    write (b, "box3.exr")
    bad.clear()

    # fillholes_pushpull
    b = test_iba (ImageBufAlgo.fillholes_pushpull, ImageBuf(OIIO_TESTSUITE_ROOT+"/oiiotool/ref/hole.tif"))
    write (b, "tahoe-filled.tif", oiio.UINT8)

    # over
    b = test_iba (ImageBufAlgo.over, ImageBuf(OIIO_TESTSUITE_ROOT+"/oiiotool-composite/src/a.exr"),
                           ImageBuf(OIIO_TESTSUITE_ROOT+"/oiiotool-composite/src/b.exr"))
    write (b, "a_over_b.exr")

    # FIXME - no test for zover (not in oiio-composite either)

    b = make_constimage (320, 240, 3, oiio.FLOAT)
    ImageBufAlgo.render_text (b, 25, 50, "Hello, world",
                              16, "DroidSerif", (1,1,1))
    ImageBufAlgo.render_text (b, 50, 120, "Go Big Red!",
                              42, "", (1,0,0))
    write (b, "text.tif", oiio.UINT8)

    b = make_constimage (320, 240, 3, oiio.FLOAT)
    broi = b.roi
    textsize = ImageBufAlgo.text_size ("Centered", 40)
    if textsize.defined :
        x = broi.xbegin + broi.width//2  - (textsize.xbegin + textsize.width//2)
        y = broi.ybegin + broi.height//2 - (textsize.ybegin + textsize.height//2)
        ImageBufAlgo.render_text (b, x, y, "Centered", 40)
    write (b, "textcentered.tif", oiio.UINT8)

    # FIXME - need tests for render_point, render_line, render_box

    # histogram, histogram_draw,
    b = make_constimage (100, 100, 3, oiio.UINT8, (.1, .2, .3))
    Rhist = ImageBufAlgo.histogram (b, channel=0, bins=4)
    Ghist = ImageBufAlgo.histogram (b, channel=1, bins=4)
    Bhist = ImageBufAlgo.histogram (b, channel=2, bins=4)
    print ("R hist: ", Rhist)
    print ("G hist: ", Ghist)
    print ("B hist: ", Bhist)

    # make_texture
    ImageBufAlgo.make_texture (oiio.MakeTxTexture,
                               ImageBuf(OIIO_TESTSUITE_ROOT+"/common/tahoe-small.tif"),
                               "tahoe-small.tx")

    # capture_image - no test

    print ("Done.")
except Exception as detail:
    print ("Unknown exception:", detail)




#!/usr/bin/env python

# Copyright Contributors to the OpenImageIO project.
# SPDX-License-Identifier: Apache-2.0
# https://github.com/AcademySoftwareFoundation/OpenImageIO


redirect = ' >> out.txt 2>&1 '

command += run_app (oiio_app("igrep") + " -i -E wg ../oiio-images/tahoe-gps.jpg")




#!/usr/bin/env python

# Copyright Contributors to the OpenImageIO project.
# SPDX-License-Identifier: Apache-2.0
# https://github.com/AcademySoftwareFoundation/OpenImageIO


command += oiiotool ("-pattern constant:color=.5,.1,.1 256x256 3 -text:size=50:x=75:y=140 u1v1 --attrib Make pet --attrib Model dog -d uint8 -otex file_u1v1.tx")
command += oiiotool ("-pattern constant:color=.1,.5,.1 256x256 3 -text:size=50:x=75:y=140 u2v1 --attrib Make pet --attrib Model dog -d uint8 -otex file_u2v1.tx")
# force DateTime to differ in these files
command += "sleep 1 ; "
command += oiiotool ("-pattern constant:color=.1,.1,.5 256x256 3 -text:size=50:x=75:y=140 u1v2 --attrib Make pet --attrib Model cat -d uint8 -otex file_u1v2.tx")
command += oiiotool ("-pattern constant:color=.1,.5,.5 256x256 3 -text:size=50:x=75:y=140 u2v2 --attrib Make pet --attrib Model dog -d uint8 -otex file_u2v2.tx")

command += oiiotool ("-pattern constant:color=.5,.1,.1 256x256 3 -text:size=50:x=75:y=140 u1v1 --attrib Make pet --attrib Model dog -d uint8 -otex file0_u0v0.tx")
command += oiiotool ("-pattern constant:color=.1,.5,.1 256x256 3 -text:size=50:x=75:y=140 u2v1 --attrib Make pet --attrib Model dog -d uint8 -otex file0_u1v0.tx")
# force DateTime to differ in these files
command += "sleep 1 ; "
command += oiiotool ("-pattern constant:color=.1,.1,.5 256x256 3 -text:size=50:x=75:y=140 u1v2 --attrib Make pet --attrib Model cat -d uint8 -otex file0_u0v1.tx")
command += oiiotool ("-pattern constant:color=.1,.5,.5 256x256 3 -text:size=50:x=75:y=140 u2v2 --attrib Make pet --attrib Model dog -d uint8 -otex file0_u1v1.tx")

command += oiiotool ("-pattern constant:color=.5,.1,.1 256x256 3 -text:size=50:x=75:y=140 u1v1 --attrib Make pet --attrib Model dog -d uint8 -otex file0.u1_v1.tx")
command += oiiotool ("-pattern constant:color=.1,.5,.1 256x256 3 -text:size=50:x=75:y=140 u2v1 --attrib Make pet --attrib Model dog -d uint8 -otex file0.u2_v1.tx")
# force DateTime to differ in these files
command += "sleep 1 ; "
command += oiiotool ("-pattern constant:color=.1,.1,.5 256x256 3 -text:size=50:x=75:y=140 u1v2 --attrib Make pet --attrib Model cat -d uint8 -otex file0.u1_v2.tx")
command += oiiotool ("-pattern constant:color=.1,.5,.5 256x256 3 -text:size=50:x=75:y=140 u2v2 --attrib Make pet --attrib Model dog -d uint8 -otex file0.u2_v2.tx")

# Textures for <uvtile>
command += oiiotool ("-pattern constant:color=.5,.1,.1 256x256 3 -text:size=50:x=75:y=140 u1v1 --attrib Make pet --attrib Model dog -d uint8 -otex file1.u0_v0.tx")
command += oiiotool ("-pattern constant:color=.1,.5,.1 256x256 3 -text:size=50:x=75:y=140 u2v1 --attrib Make pet --attrib Model dog -d uint8 -otex file1.u1_v0.tx")
# force DateTime to differ in these files
command += "sleep 1 ; "
command += oiiotool ("-pattern constant:color=.1,.1,.5 256x256 3 -text:size=50:x=75:y=140 u1v2 --attrib Make pet --attrib Model cat -d uint8 -otex file1.u0_v1.tx")
command += oiiotool ("-pattern constant:color=.1,.5,.5 256x256 3 -text:size=50:x=75:y=140 u2v2 --attrib Make pet --attrib Model dog -d uint8 -otex file1.u1_v1.tx")

command += testtex_command ("\"file_<U><V>.tx\"",
                            "-nowarp -scalest 2 2 --iters 100 -res 128 128 -d uint8 -o out.tif")
#command += diff_command ("out.tif", "ref/out.tif", "--fail 0.0005 --warn 0.0005")

# Test get_texture_info on one thing that should be identical across all
# the tiles, and one thing that should not be.
command += testtex_command ("\"file_<U><V>.tx\" --gettextureinfo Make --iters 0")
command += testtex_command ("\"file_<U><V>.tx\" --gettextureinfo Model --iters 0")

# Test the other forms of 2-index udim patterns
command += testtex_command ("\"file0_u##v##.tx\"",
                            "-nowarp -scalest 2 2 --iters 100 -res 128 128 -d uint8 -o out2.tif")
command += testtex_command ("\"file0_u##v##.tx\" --gettextureinfo Make --iters 0")
command += testtex_command ("\"file0_u##v##.tx\" --gettextureinfo Model --iters 0")

command += testtex_command ("\"file0_<u><v>.tx\"",
                            "-nowarp -scalest 2 2 --iters 100 -res 128 128 -d uint8 -o out3.tif")
command += testtex_command ("\"file0_<u><v>.tx\" --gettextureinfo Make --iters 0")
command += testtex_command ("\"file0_<u><v>.tx\" --gettextureinfo Model --iters 0")

command += testtex_command ("\"file0.<UVTILE>.tx\"",
                            "-nowarp -scalest 2 2 --iters 100 -res 128 128 -d uint8 -o out4.tif")
command += testtex_command ("\"file0.<UVTILE>.tx\" --gettextureinfo Make --iters 0")
command += testtex_command ("\"file0.<UVTILE>.tx\" --gettextureinfo Model --iters 0")

command += testtex_command ("\"file1.<uvtile>.tx\"",
                            "-nowarp -scalest 2 2 --iters 100 -res 128 128 -d uint8 -o out5.tif")
command += testtex_command ("\"file1.<uvtile>.tx\" --gettextureinfo Make --iters 0")
command += testtex_command ("\"file1.<uvtile>.tx\" --gettextureinfo Model --iters 0")

outputs = [ "out.tif", "out2.tif", "out3.tif", "out4.tif", "out5.tif", "out.txt" ]



#!/usr/bin/env python

# Copyright Contributors to the OpenImageIO project.
# SPDX-License-Identifier: Apache-2.0
# https://github.com/AcademySoftwareFoundation/OpenImageIO


command += testtex_command ("--maketests 10 \"test{:04}.exr\" --maketest-res 1024 --threadtimes 7 --wedge")

outputs = [ ]



#!/usr/bin/env python

# Copyright Contributors to the OpenImageIO project.
# SPDX-License-Identifier: Apache-2.0
# https://github.com/AcademySoftwareFoundation/OpenImageIO


redirect = " >> stats.txt"
command = testtex_command ("../common/textures/grid.tx -res 64 48 -v --teststatquery --runstats")
outputs = [ "out.exr" ]



#!/usr/bin/env python

# Copyright Contributors to the OpenImageIO project.
# SPDX-License-Identifier: Apache-2.0
# https://github.com/AcademySoftwareFoundation/OpenImageIO

files = [ "dpx_nuke_10bits_rgb.dpx", "dpx_nuke_16bits_rgba.dpx" ]
for f in files:
    command += rw_command (OIIO_TESTSUITE_IMAGEDIR, f)


# Additionally, test for regressions for endian issues with 16 bit DPX output
# (related to issue #354)
command += oiio_app("oiiotool") + " src/input_rgb_mattes.tif -o output_rgb_mattes.dpx >> out.txt;"
command += oiio_app("idiff") + " src/input_rgb_mattes.tif output_rgb_mattes.dpx >> out.txt;"

# Test reading and writing of stereo DPX (multi-image)
#command += (oiio_app("oiiotool") + "--create 80x60 3 --text:x=10 Left "
#            + "--caption \"view angle: left\" -d uint10 -o L.dpx >> out.txt;")
#command += (oiio_app("oiiotool") + "--create 80x60 3 --text:x=10 Right "
#            + "--caption \"view angle: right\" -d uint10 -o R.dpx >> out.txt;")
command += (oiio_app("oiiotool") + "ref/L.dpx ref/R.dpx --siappend -o stereo.dpx >> out.txt;")
command += info_command("stereo.dpx", safematch=True, hash=False, extraargs="--stats")
command += oiio_app("idiff") + "-a stereo.dpx ref/stereo.dpx >> out.txt;"

# Test read/write of 1-channel DPX -- take a color image, make it grey,
# write it as 1-channel DPX, then read it again and compare to a reference.
# The reference is stored as TIFF rather than DPX just because it has
# fantastically better compression.
command += oiiotool(OIIO_TESTSUITE_IMAGEDIR+"/dpx_nuke_16bits_rgba.dpx"
                    " -chsum:weight=0.333,0.333,0.333 -chnames Y -ch Y -o grey.dpx")
command += info_command("grey.dpx", safematch=True)
command += diff_command("grey.dpx", "ref/grey.tif")



#!/usr/bin/env python

# Copyright Contributors to the OpenImageIO project.
# SPDX-License-Identifier: Apache-2.0
# https://github.com/AcademySoftwareFoundation/OpenImageIO


from __future__ import print_function
from __future__ import absolute_import
import numpy
import OpenImageIO as oiio


# Print the contents of an ImageSpec
def print_imagespec (spec, msg="") :
    if msg != "" :
        print (str(msg))
    print ("  resolution (width,height,depth) = ", spec.width, spec.height, spec.depth)
    print ("  origin (x,y,z) = ", spec.x, spec.y, spec.z)
    print ("  full res = ", spec.full_width, spec.full_height, spec.full_depth)
    print ("  full origin = ", spec.full_x, spec.full_y, spec.full_z)
    if spec.tile_width :
        print ("  tile size = ", spec.tile_width, spec.tile_height, spec.tile_depth)
    else :
        print ("  untiled")
    print ("  format = ", str(spec.format))
    print ("  nchannels = ", spec.nchannels)
    print ("  channelformats = ", spec.channelformats)
    print ("  channel names = ", spec.channelnames)
    print ("  alpha channel = ", spec.alpha_channel)
    print ("  z channel = ", spec.z_channel)
    print ("  deep = ", spec.deep)
    # print ("  quantization: black=%d, white=%d, min=%d, max=%d" 
    #        % (spec.quant_black, spec.quant_white, spec.quant_min, spec.quant_max))
    for i in range(len(spec.extra_attribs)) :
        print ("  ", spec.extra_attribs[i].name, "=", spec.extra_attribs[i].value)
    print ()



######################################################################
# main test starts here

try:
    # test contstructors
    print_imagespec (oiio.ImageSpec(), "ctr: null")
    print_imagespec (oiio.ImageSpec(oiio.UINT16), "ctr: UINT16")
    print_imagespec (oiio.ImageSpec(640, 480, 3, oiio.UINT8), "ctr: 640x480,3 UINT8")

    # test setting fields
    s = oiio.ImageSpec()
    s.set_format (oiio.FLOAT)
    s.x = 1
    s.y = 2
    s.z = 3
    s.width = 640
    s.height = 480
    s.depth = 1
    s.full_x = 4
    s.full_y = 5
    s.full_z = 6
    s.full_width = 1280
    s.full_height = 960
    s.full_depth = 7
    s.tile_width = 32
    s.tile_height = 64
    s.tile_depth = 16
    s.nchannels = 5;
    s.alpha_channel = 3;
    s.z_channel = 4;
    s.default_channel_names()
    s.channelformats = (oiio.UINT8, oiio.UINT8, oiio.UINT8, oiio.UINT8, oiio.FLOAT)
    print_imagespec (s, "lots of fields")
    print (" B channel =", s.channelindex("B"))
    print (" foo channel =", s.channelindex("foo"))

    print ("channel bytes =", s.channel_bytes())
    print ("  channel_bytes(1) =", s.channel_bytes(1), "native", s.channel_bytes(1,True))
    print ("  channel_bytes(4) =", s.channel_bytes(4), "native", s.channel_bytes(4,True))
    print ("pixel bytes =", s.pixel_bytes(), "native", s.pixel_bytes(True))
    print ("scanline bytes =", s.scanline_bytes(), "native", s.scanline_bytes(True))
    print ("tile bytes =", s.tile_bytes(), "native", s.tile_bytes(True))
    print ("image bytes =", s.image_bytes(), "native", s.image_bytes(True))
    print ("tile pixels =", s.tile_pixels())
    print ("image_pixels =", s.image_pixels())
    print ("size_t_safe =", s.size_t_safe())
    print ("get_channelformats:", s.get_channelformats())
    print ("channelindex('G') =", s.channelindex('G'))
    print ("channel_name(1) =", s.channel_name(1))
    print ("channelformat(1) =", s.channelformat(1))
    print ()

    s.attribute ("foo_str", "blah")
    s.attribute ("foo_int", 14)
    s.attribute ("foo_float", 3.14)
    s.attribute ("foo_vector", oiio.TypeDesc.TypeVector, (1, 0, 11))
    s.attribute ("foo_matrix", oiio.TypeDesc.TypeMatrix,
                 (1, 0, 0, 0, 0, 2, 0, 0, 0, 0, 1, 0, 1, 2, 3, 1))
    s.attribute ("smpte:TimeCode", oiio.TypeDesc.TypeTimeCode, (18356486, 4294967295))
    s.attribute ("ucarr", "uint8[10]", numpy.array([49, 50, 51, 0, 0, 97, 98, 99, 1, 88], dtype='B'))
    s["delfoo_str"] =  "egg"
    s["delfoo_int"] = 29
    s["delfoo_float"] = 99.5
    print ("get_int_attribute('foo_int') retrieves", s.get_int_attribute ("foo_int"))
    print ("get_int_attribute('foo_int',21) with default retrieves", s.get_int_attribute ("foo_int", 21))
    print ("get_int_attribute('foo_no',23) retrieves", s.get_int_attribute ("foo_no", 23))
    print ("get_float_attribute('foo_float') retrieves", s.get_float_attribute ("foo_float"))
    print ("get_float_attribute('foo_float_no') retrieves", s.get_float_attribute ("foo_float_no"))
    print ("get_float_attribute('foo_float_no',2.7) retrieves", s.get_float_attribute ("foo_float_no", 2.7))
    print ("get_string_attribute('foo_str') retrieves", s.get_string_attribute ("foo_str"))
    print ("get_string_attribute('foo_str_no') retrieves", s.get_string_attribute ("foo_str_no"))
    print ("get_string_attribute('foo_str_no','xx') retrieves", s.get_string_attribute ("foo_str_no", "xx"))
    print ()
    print ("getattribute('foo_int') retrieves", s.getattribute("foo_int"))
    print ("getattribute('foo_float') retrieves", s.getattribute("foo_float"))
    print ("getattribute('foo_str') retrieves", s.getattribute("foo_str"))
    print ("getattribute('foo_vector') retrieves", s.getattribute("foo_vector"))
    print ("getattribute('foo_matrix') retrieves", s.getattribute("foo_matrix"))
    print ("getattribute('foo_no') retrieves", s.getattribute("foo_no"))
    print ("getattribute('smpte:TimeCode') retrieves", s.getattribute("smpte:TimeCode"))
    print ("getattribute('ucarr') retrieves", s.getattribute("ucarr"))
    print ("getattribute('unknown') retrieves", s.getattribute("unknown"))
    print ("s.get('foo_int') =", s.get('foo_int'))
    print ("s.get('ucarr') retrieves", s.get("ucarr"))
    try :
        print ("s['ucarr'] retrieves", s['ucarr'])
    except KeyError :
        print ("s['ucarr'] not found")
    print ("s.get('unknown') =", s.get('unknown'))
    print ("s.get('unknown', 123) =", s.get('unknown'))
    print ("s['delfoo_float'] =", s['delfoo_float'])
    print ("s['delfoo_int'] =", s['delfoo_int'])
    print ("s['delfoo_str'] =", s['delfoo_str'])
    try :
        print ("s['unknown'] =", s['unknown'])
    except KeyError :
        print ("s['unknown'] raised a KeyError (as expected)")
    except :
        print ("s['unknown'] threw an unknown exception (oh no!)")
    print("'foo_int' in s =", "foo_int" in s)
    print("'unknown' in s =", "unknown" in s)
    s["extra"] = 1  # add 'extra', then delete it
    del s["extra"]  # it should not appear in the serialization below
    print ()

    print ("extra_attribs size is", len(s.extra_attribs))
    for i in range(len(s.extra_attribs)) :
        print (i, s.extra_attribs[i].name, s.extra_attribs[i].type, s.extra_attribs[i].value)
        print (s.metadata_val (s.extra_attribs[i], True))
    print ()
    print ("seralize(xml):")
    print (s.serialize("xml"))
    print ("serialize(text, human):")
    print (s.serialize("text", "detailedhuman"))
    print ()

    s.attribute("dog", "Spot")
    print ("Added dog: ", s.getattribute("dog"))
    s.erase_attribute("dog")
    print ("After erasing dog, dog = ", s.getattribute("dog"))
    print()

    # test initialization from ROI
    print ("Testing construction from ROI:")
    sroi = oiio.ImageSpec (oiio.ROI(0,640,0,480,0,1,0,3), oiio.FLOAT);
    print_imagespec (sroi)

    print ("\nTesting set_colorspace:")
    s = oiio.ImageSpec()
    s.set_colorspace("sRGB")
    print ("  after set_colorspace('sRGB'):", s.get_string_attribute("oiio:ColorSpace"))
    s.set_colorspace("")
    print ("  after set_colorspace(''):", s.get_string_attribute("oiio:ColorSpace"))

    # Also test global OIIO functions here
    print ("\nTesting global attribute store/retrieve:")
    oiio.attribute ("plugin_searchpath", "perfect")
    print ("get_string_attribute plugin_searchpath : ", oiio.get_string_attribute ("plugin_searchpath", "bad"))
    print ("get_int_attribute plugin_searchpath : ", oiio.get_int_attribute ("plugin_searchpath", 0))
    print ("getattribute TypeString plugin_searchpath : ", oiio.getattribute ("plugin_searchpath", oiio.TypeDesc.TypeString))
    print ("getattribute TypeFloat plugin_searchpath : ", oiio.getattribute ("plugin_searchpath", oiio.TypeDesc.TypeFloat))
    print ("getattribute TypeString blahblah : ", oiio.getattribute ("blahblah", oiio.TypeDesc.TypeString))

    print ("Done.")
except Exception as detail:
    print ("Unknown exception:", detail)




#!/usr/bin/env python

# Copyright Contributors to the OpenImageIO project.
# SPDX-License-Identifier: Apache-2.0
# https://github.com/AcademySoftwareFoundation/OpenImageIO


# Make an .hdr image out of an openexr image -- tests write
command += oiiotool (OIIO_TESTSUITE_IMAGEDIR+"/ScanLines/MtTamWest.exr -o MtTamWest.hdr")
# read and print stats, that tests the read
command += oiiotool ("-v -info -stats MtTamWest.hdr")
# To double check, get stats on the original for comparison -- should be close
command += oiiotool ("-stats " + OIIO_TESTSUITE_IMAGEDIR+"/ScanLines/MtTamWest.exr")



#!/usr/bin/env python

# Copyright Contributors to the OpenImageIO project.
# SPDX-License-Identifier: Apache-2.0
# https://github.com/AcademySoftwareFoundation/OpenImageIO


from __future__ import absolute_import
import os

# print("ociover =", ociover)

# Make test pattern with increasing intensity left to right, decreasing
# alpha going down. Carefully done so that the first pixel is 0.0, last
# pixel is 1.0 (correcting for the half pixel offset).
command += oiiotool ("-pattern fill:topleft=0,0,0,1:topright=1,1,1,1:bottomleft=0,0,0,0:bottomright=1,1,1,0 256x256 4 "
                     + " -d uint8 -o greyalpha_lin_srgb.tif")
command += oiiotool ("-pattern fill:topleft=0,0,0:topright=1,1,1:bottomleft=0,0,0:bottomright=1,1,1 256x256 3 "
                     + " -d uint8 -o grey_lin_srgb.tif")


# test --colormap
command += oiiotool ("--autocc " + "../common/tahoe-tiny.tif" +
                     " --colormap inferno " +
                     "-d uint8 -o colormap-inferno.tif")
command += oiiotool ("--autocc " + "../common/tahoe-tiny.tif" +
                     " --colormap .25,.25,.25,0,.5,0,1,0,0 " +
                     "-d uint8 -o colormap-custom.tif")

colormaps = [ "magma", "inferno", "plasma", "viridis", "turbo", "blue-red", "spectrum", "heat" ]
for c in colormaps :
    command += oiiotool ("--pattern fill:left=0,0,0:right=1,1,1 64x64 3" +
                         " --colormap " + c +
                         " -d uint8 -o cmap-" + c + ".tif")

# test unpremult/premult
command += oiiotool ("--pattern constant:color=.1,.1,.1,1 100x100 4 " 
            + " --fill:color=.2,.2,.2,.5 30x30+50+50 "
            + " -d half -o premulttarget.exr")
command += oiiotool ("premulttarget.exr --unpremult -o unpremult.exr")
command += oiiotool ("unpremult.exr --premult -o premult.exr")

# test --no-autopremult on a TGA file thet needs it.
command += oiiotool ("--no-autopremult src/rgba.tga --ch R,G,B -o rgbfromtga.png")

# test --contrast
command += oiiotool ("--autocc " + "../common/tahoe-tiny.tif" +
                     " -contrast:black=0.1:white=0.75 -d uint8 -o contrast-stretch.tif")
command += oiiotool ("--autocc " + "../common/tahoe-tiny.tif" +
                     " -contrast:min=0.1:max=0.75 -d uint8 -o contrast-shrink.tif")
command += oiiotool ("--autocc " + "../common/tahoe-tiny.tif" +
                     " -contrast:black=1:white=0 -d uint8 -o contrast-inverse.tif")
command += oiiotool ("--autocc " + "../common/tahoe-tiny.tif" +
                     " -contrast:black=1,1,.25:white=1,1,0.25 -d uint8 -o contrast-threshold.tif")
command += oiiotool ("--autocc " + "../common/tahoe-tiny.tif" +
                     " -contrast:scontrast=5 -d uint8 -o contrast-sigmoid5.tif")

# test --saturate
command += oiiotool ("--autocc " + "../common/tahoe-tiny.tif" +
                     " --saturate 0 -d uint8 -o tahoe-sat0.tif")
command += oiiotool ("--autocc " + "../common/tahoe-tiny.tif" +
                     " --saturate 2 -d uint8 -o tahoe-sat2.tif")



#
# Test basic color transformation / OCIO functionality
#

# colorconvert without unpremult
if float(ociover) >= 2.2 :
    command += oiiotool ("greyalpha_lin_srgb.tif --colorconvert:unpremult=0 lin_srgb sRGB -o greyalpha_sRGB.tif")
    command += oiiotool ("greyalpha_lin_srgb.tif --colorconvert:unpremult=1 lin_srgb sRGB -o greyalpha_sRGB_un.tif")
    command += oiiotool ("grey_lin_srgb.tif --colorconvert:unpremult=0 lin_srgb sRGB -o grey_sRGB.tif")
    command += oiiotool ("grey_lin_srgb.tif --colorconvert:unpremult=1 lin_srgb sRGB -o grey_sRGB_un.tif")
else:
    command += oiiotool ("greyalpha_lin_srgb.tif --colorconvert:unpremult=0 linear sRGB -o greyalpha_sRGB.tif")
    command += oiiotool ("greyalpha_lin_srgb.tif --colorconvert:unpremult=0 linear Cineon -o greyalpha_Cineon.tif")
    command += oiiotool ("greyalpha_lin_srgb.tif --colorconvert:unpremult=1 linear sRGB -o greyalpha_sRGB_un.tif")
    command += oiiotool ("greyalpha_lin_srgb.tif --colorconvert:unpremult=1 linear Cineon -o greyalpha_Cineon_un.tif")
    command += oiiotool ("grey_lin_srgb.tif --colorconvert:unpremult=0 linear sRGB -o grey_sRGB.tif")
    command += oiiotool ("grey_lin_srgb.tif --colorconvert:unpremult=1 linear sRGB -o grey_sRGB_un.tif")
 
# test color convert by matrix
command += oiiotool ("--autocc " + "../common/tahoe-tiny.tif"+
                     " "
                     + "--ccmatrix 0.805,0.506,-0.311,0,-0.311,0.805,0.506,0,0.506,-0.311,0.805,0,0,0,0,1 "
                     + "-d uint8 -o tahoe-ccmatrix.tif")

# Apply a display
if float(ociover) >= 2.2 :
    command += oiiotool ("greyalpha_lin_srgb.tif --iscolorspace lin_srgb --ociodisplay \"sRGB - Display\" Un-tone-mapped -o display-sRGB.tif")
else :
    command += oiiotool ("greyalpha_lin_srgb.tif --ociodisplay default sRGB -o display-sRGB.tif")

# Applying a look
if float(ociover) >= 2.2 :
    command += oiiotool ("--autocc ../common/tahoe-tiny.tif --ociolook \"ACES 1.3 Reference Gamut Compression\" -o look-default.tif")

# TODO: should test applying a file transform

# test various behaviors and misbehaviors related to OCIO configs.
command += oiiotool ("--nostderr --colorconfig missing.ocio -echo \"Nonexistent config\"", failureok=True)

#   What happens when we read an OCIOv2 config? In particular, when building
#   against OCIOv1, we should at worst have an error message, not crash with
#   an uncaught exception.
command += oiiotool ("--nostderr --colorconfig ../common/OpenColorIO/ocio-v2_demo.ocio -echo \"OCIOv2 config Ok\"", failureok=True)


# To add more tests, just append more lines like the above and also add
# the new 'feature.tif' (or whatever you call it) to the outputs list,
# below.


# Outputs to check against references
outputs = [
            "colormap-inferno.tif", "colormap-custom.tif",
            "unpremult.exr", "premult.exr",
            "contrast-stretch.tif",
            "contrast-shrink.tif",
            "contrast-inverse.tif",
            "contrast-threshold.tif",
            "contrast-sigmoid5.tif",
            "display-sRGB.tif",
            "rgbfromtga.png",
            "greyalpha_sRGB.tif",
            "greyalpha_sRGB_un.tif",
            "grey_sRGB.tif",
            "grey_sRGB_un.tif",
            "tahoe-ccmatrix.tif",
            "tahoe-sat0.tif",
            "tahoe-sat2.tif"
    ]
for c in colormaps :
    outputs += [ "cmap-" + c + ".tif" ]
if float(ociover) >= 2.2 :
    outputs += [ "look-default.tif" ]
else :
    outputs += [ 
            "greyalpha_Cineon.tif",
            "greyalpha_Cineon_un.tif",
    ]
outputs += [ "out.txt" ]



#!/usr/bin/env python

# Copyright Contributors to the OpenImageIO project.
# SPDX-License-Identifier: BSD-3-Clause and Apache-2.0
# https://github.com/AcademySoftwareFoundation/OpenImageIO

# save the error output
redirect = " >> out.txt 2>&1 "

files = [ "oiio-logo-no-alpha.png",  "oiio-logo-with-alpha.png" ]
for f in files:
        command += rw_command (OIIO_TESTSUITE_IMAGEDIR,  f)

# test writing of exif data
command += oiiotool ("--create 64x64 3 " +
                     "--attrib Exif:WhiteBalance 0 " +
                     "--attrib Exif:FocalLength 45.7 " +
                     "-o exif.png")
command += info_command ("exif.png", safematch=True)

# regression test for 16 bit output bug
command += oiiotool ("--pattern fill:topleft=1,0,0,1:topright=0,1,0,1:bottomleft=0,0,1,1:bottomright=1,1,1,1 16x16 4 -d uint16 -o test16.png")

# regression test for wrong gamma correction for partial alpha
command += oiiotool ("src/alphagamma.png --printinfo:stats=1")

# Test high quality alpha deassociation using alpha value close to zero.
# This example is inspired by Yafes on the Slack.
command += oiiotool ("--pattern fill:color=0.00235,0.00106,0.00117,0.0025 1x1 4 -d uint8 -o smallalpha.png")
command += oiiotool ("--no-autopremult --dumpdata smallalpha.png")

outputs = [ "test16.png", "out.txt" ]




#!/usr/bin/env python

# Copyright Contributors to the OpenImageIO project.
# SPDX-License-Identifier: Apache-2.0
# https://github.com/AcademySoftwareFoundation/OpenImageIO

redirect = " >> out.txt 2>&1"

imagedir = OIIO_TESTSUITE_IMAGEDIR + "/targa"

files = [ "CBW8.TGA", "CCM8.TGA", "CTC16.TGA", "CTC24.TGA", "CTC32.TGA",
          "UBW8.TGA", "UCM8.TGA", "UTC16.TGA", "UTC24.TGA", "UTC32.TGA",
          "round_grill.tga" ]
for f in files:
    command += rw_command (imagedir, f)


# Test corrupted files
command += iconvert("-v src/crash1.tga crash1.exr", failureok = True)
command += oiiotool("--oiioattrib try_all_readers 0 src/crash2.tga -o crash2.exr", failureok = True)
command += oiiotool("--oiioattrib try_all_readers 0 src/crash3.tga -o crash3.exr", failureok = True)
command += oiiotool("--oiioattrib try_all_readers 0 src/crash4.tga -o crash4.exr", failureok = True)
command += oiiotool("--oiioattrib try_all_readers 0 src/crash5.tga -o crash5.exr", failureok = True)
command += oiiotool("--oiioattrib try_all_readers 0 src/crash6.tga -o crash6.exr", failureok = True)
command += oiiotool("--oiioattrib try_all_readers 0 src/crash1707.tga -o crash1707.exr", failureok = True)
command += oiiotool("--oiioattrib try_all_readers 0 src/crash1708.tga -o crash1708.exr", failureok = True)
command += oiiotool("--oiioattrib limits:imagesize_MB 1024 "
                    "--oiioattrib try_all_readers 0 "
                    "src/crash3952.tga -o crash3952.exr", failureok = True)

# Test odds and ends, unusual files
command += rw_command("src", "1x1.tga")

outputs += [ 'out.txt' ]



#!/usr/bin/env python

# Copyright Contributors to the OpenImageIO project.
# SPDX-License-Identifier: Apache-2.0
# https://github.com/AcademySoftwareFoundation/OpenImageIO

failureok = 1
redirect = ' >> out.txt 2>&1 '
files = [
    "dds_bc1.dds",
    "dds_bc1_mips.dds",
    "dds_bc2.dds",
    "dds_bc3.dds",
    "dds_bc4.dds",
    "dds_bc5.dds",
    "dds_bc6hu.dds",
    "dds_bc6hu_hdr.dds",
    "dds_bc7.dds",
    "dds_bgr8.dds",
    "dds_npot_bc3.dds",
    "dds_npot_bc3_mips.dds",
    "dds_npot_rgba8.dds",
    "dds_npot_rgba8_mips.dds",
    "dds_r5g6b5.dds",
    "dds_rgb8.dds",
    "dds_rgba4.dds",
    "dds_rgba8.dds",
    "dds_rgba8_mips.dds",
    "dds_a8.dds",
    "dds_abgr8.dds",
    "dds_bc3nm.dds",
    "dds_bc3rxgb.dds",
    "dds_bc3ycocg.dds",
    "dds_l8.dds",
    "dds_l8a8.dds",
    "dds_rgb10a2.dds",
    "dds_rgb332.dds",
    "dds_rgb5a1.dds",
    "dds_dxgi_bc1_srgb.dds",
    "dds_dxgi_bc2_srgb.dds",
    "dds_dxgi_bc3_srgb.dds",
    "dds_dxgi_bc7_srgb.dds",
    "dds_dxgi_rgba8_srgb.dds",
    "dds_dxgi_bgra8_srgb.dds",
    "dds_dxgi_bgrx8_srgb.dds",
    "dds_dxgi_rgb10a2.dds",
    "dds_dxgi_r16.dds",

    # In some CI matrix cases a PtexReader
    # also attempts to read this, and outputs
    # a "PtexReader error: read failed (EOF)"
    # failure error.
    #"broken/dds_8bytes.dds",
    "broken/dds_bc3_just_header.dds",
    "broken/dds_bc3_no_full_header.dds",
    "broken/dds_bc7_just_header.dds",
    "broken/dds_bc7_not_enough_data.dds" ]
for f in files:
    command += info_command (OIIO_TESTSUITE_IMAGEDIR + "/" + f)

# Test more corrupted files or those that used to crash
command += info_command ("src/crash-1634.dds", hash=True)
command += info_command ("src/crash-1635.dds", hash=True)
command += info_command ("src/crash-3950.dds", hash=True)



#!/usr/bin/env python

# Copyright Contributors to the OpenImageIO project.
# SPDX-License-Identifier: Apache-2.0
# https://github.com/AcademySoftwareFoundation/OpenImageIO

# ../openexr-images/Chromaticities:
# README         Rec709.exr     Rec709_YC.exr  XYZ.exr        XYZ_YC.exr
imagedir = OIIO_TESTSUITE_IMAGEDIR + "/Chromaticities"
files = [
    "Rec709.exr", "XYZ.exr"
]
for f in files:
    command += rw_command (imagedir, f)

# ../openexr-images/LuminanceChroma:
# CrissyField.exr  Garden.exr       StarField.exr
# Flowers.exr      MtTamNorth.exr
imagedir = OIIO_TESTSUITE_IMAGEDIR + "/LuminanceChroma"
command += rw_command (imagedir, "Garden.exr")



#!/usr/bin/env python

# Copyright Contributors to the OpenImageIO project.
# SPDX-License-Identifier: Apache-2.0
# https://github.com/AcademySoftwareFoundation/OpenImageIO


command += oiiotool("../common/grid.tif --crop 512x512+200+100 -o grid-crop.tif")
command += maketx_command ("grid-crop.tif", "grid-crop.tx")
command += testtex_command ("grid-crop.tx", "--wrap black")

outputs = [ "out.exr" ]



#!/usr/bin/env python 

# Copyright Contributors to the OpenImageIO project.
# SPDX-License-Identifier: Apache-2.0
# https://github.com/AcademySoftwareFoundation/OpenImageIO

# Set up some udims
command += oiiotool ("-pattern constant:color=.5,.1,.1 256x256 3 -text:size=50:x=75:y=140 1001 --attrib Make pet --attrib Model dog -d uint8 -otex file.1001.tx")
command += oiiotool ("-pattern constant:color=.1,.5,.1 256x256 3 -text:size=50:x=75:y=140 1002 --attrib Make pet --attrib Model dog -d uint8 -otex file.1002.tx")
command += oiiotool ("-pattern constant:color=.1,.1,.5 256x256 3 -text:size=50:x=75:y=140 1011 --attrib Make pet --attrib Model cat -d uint8 -otex file.1011.tx")
command += oiiotool ("-pattern constant:color=.1,.5,.5 256x256 3 -text:size=50:x=75:y=140 1012 --attrib Make pet --attrib Model dog -d uint8 -otex file.1012.tx")
command += oiiotool ("-pattern constant:color=.1,.5,.5 256x256 3 -text:size=50:x=75:y=140 1012 --attrib Make pet --attrib Model dog -d uint8 -otex file.1032.tx")

command += pythonbin + " src/test_texture_sys.py > out.txt"




#!/usr/bin/env python

# Copyright Contributors to the OpenImageIO project.
# SPDX-License-Identifier: Apache-2.0
# https://github.com/AcademySoftwareFoundation/OpenImageIO


command += pythonbin + " src/test_imagespec.py > out.txt"




#!/usr/bin/env python

# Copyright Contributors to the OpenImageIO project.
# SPDX-License-Identifier: Apache-2.0
# https://github.com/AcademySoftwareFoundation/OpenImageIO


from __future__ import print_function
from __future__ import absolute_import
import array
import numpy
import OpenImageIO as oiio



######################################################################
# main test starts here

try:
    ic = oiio.ImageCache()

    # Set some attributes
    ic.attribute("max_open_files", 90)
    ic.attribute("max_memory_MB", 900.0)
    ic.attribute("searchpath", "../common")
    print ("getattribute(\"max_open_files\")", ic.getattribute("max_open_files"))
    print ("getattribute(\"max_memory_MB\")", ic.getattribute("max_memory_MB"))
    print ("getattribute(\"searchpath\")", ic.getattribute("searchpath"))

    # Force a file to be touched by the IC and test get_imagespec
    spec = ic.get_imagespec("../common/tahoe-tiny.tif")
    print ("tahoe_tiny is", spec.width, "x", spec.height)
    spec = ic.get_imagespec("../common/grid.tif")
    print ("grid is", spec.width, "x", spec.height)

    # Test getattribute(name, type) with the full type specified
    print ("full getattribute stat:cache_memory_used", ic.getattribute("stat:cache_memory_used", 'int64'))
    print ("full getattribute stat:image_size", ic.getattribute("stat:image_size", 'int64'))
    total_files = ic.getattribute("total_files", 'int')
    print ("full getattribute total_files", ic.getattribute("total_files", 'int'))
    print ("full getattribute all_filenames", ic.getattribute("all_filenames", 'string[{}]'.format(total_files)))

    # Test getattributetype() to retrieve the type of an attribute
    print ("getattributetype stat:cache_memory_used", ic.getattributetype("stat:cache_memory_used"))
    print ("getattributetype stat:image_size", ic.getattributetype("stat:image_size"))
    print ("getattributetype total_files", ic.getattributetype("total_files"))
    print ("getattributetype all_filenames", ic.getattributetype("all_filenames"))

    # Test getattribute(name) with the type inferred from the attribute
    print ("untyped getattribute stat:cache_memory_used", ic.getattribute("stat:cache_memory_used"))
    print ("untyped getattribute stat:image_size", ic.getattribute("stat:image_size"))
    print ("untyped getattribute total_files", ic.getattribute("total_files"))
    print ("untyped getattribute all_filenames", ic.getattribute("all_filenames"))

    # Test getpixels()
    print ("getpixels from grid.tif:", ic.get_pixels("../common/grid.tif", 0, 0,
                                                     3, 5, 3, 5, 0, 1, "float"))
    print ("  has_error?", ic.has_error)
    print ("  geterror?", ic.geterror())
    print ("getpixels from broken.tif:", ic.get_pixels("broken.tif", 0, 0,
                                                     3, 5, 3, 5, 0, 1, "float"))
    print ("  has_error?", ic.has_error)
    print ("  geterror?", ic.geterror())

    print ("getstats beginning:")
    print (ic.getstats().split()[0:3])

    # invalidate some things
    ic.invalidate("../common/tahoe-tiny.tif")
    ic.invalidate_all(True)

    print ("\nDone.")
except Exception as detail:
    print ("Unknown exception:", detail)




#!/usr/bin/env python

# Copyright Contributors to the OpenImageIO project.
# SPDX-License-Identifier: Apache-2.0
# https://github.com/AcademySoftwareFoundation/OpenImageIO


# Run the script 
command += pythonbin + " src/test_imagecache.py > out.txt ;"

# compare the outputs
outputs = [ "out.txt" ]




#!/usr/bin/env python

# Copyright Contributors to the OpenImageIO project.
# SPDX-License-Identifier: BSD-3-Clause and Apache-2.0
# https://github.com/AcademySoftwareFoundation/OpenImageIO

redirect = ' >> out.txt 2>&1 '

files = [ "psd_123.psd", "psd_123_nomaxcompat.psd", "psd_bitmap.psd",
          "psd_indexed_trans.psd", "psd_rgb_8.psd", "psd_rgb_16.psd",
          "psd_rgb_32.psd", "psd_rgba_8.psd" ]
for f in files:
    command += info_command (OIIO_TESTSUITE_IMAGEDIR + "/" + f)

# Test unassociated alpha metadata
command += info_command (OIIO_TESTSUITE_IMAGEDIR + "/psd_123.psd", extraargs="--no-autopremult")

command += info_command ("src/different-mask-size.psd")
command += info_command ("src/layer-mask.psd")

# This file has a corrupted Exif block
command += info_command ("src/crash-psd-exif-1632.psd", failureok = 1)
# Corrupted thumbnail clobbered memory
command += info_command ("src/crash-thumb-1626.psd", failureok = 1)

# Test more modern (Photoshop 2023 files) with 16- and 32-bit files containing multiple sublayers
command += info_command ("src/Layers_8bit_RGB.psd")
command += info_command ("src/Layers_16bit_RGB.psd")
command += info_command ("src/Layers_32bit_RGB.psd")


#!/usr/bin/env python

# Copyright Contributors to the OpenImageIO project.
# SPDX-License-Identifier: Apache-2.0
# https://github.com/AcademySoftwareFoundation/OpenImageIO


command += maketx_command  ("../common/textures/grid.tx", "grid-half.exr",
                            "-d half", showinfo=True)
command += testtex_command ("grid-half.exr")
outputs = [ "out.exr" ]



#!/usr/bin/env python 

# Copyright Contributors to the OpenImageIO project.
# SPDX-License-Identifier: Apache-2.0
# https://github.com/AcademySoftwareFoundation/OpenImageIO

failureok = 1

outputs = [ ]

# Just for simplicity, make a checkerboard with a solid alpha
command += oiiotool (" --pattern checker 128x128 4 --ch R,G,B,=1.0"
            + " -d uint8 -o " + make_relpath("checker.tif") )
# Noise bump pattern
command += oiiotool (" --pattern noise 64x64 1"
           + " -d half -o " + make_relpath("noise.exr"))

# Basic test - recreate the grid texture
command += maketx_command ("../common/grid.tif", "grid.tx", showinfo=True)

# Make tex twice using -u
command += maketx_command ("checker.tif", "checker.tx", "-u", showinfo=True)
command += maketx_command ("checker.tif", "checker.tx", "-u", showinfo=True)


# Test --resize (to power of 2) with the grid, which is 1000x1000
command += maketx_command ("../common/grid.tif", "grid-resize.tx",
                           "--resize", showinfo=True)

# Test -d to set output data type
command += maketx_command ("checker.tif", "checker-uint16.tx",
                           "-d uint16", showinfo=True)

# Test --nchannels to restrict the number of channels
command += maketx_command ("checker.tif", "checker-1chan.tx",
                           "--nchannels 1", showinfo=True)

# Test --tiles to set a non-default tile size
command += maketx_command ("checker.tif", "checker-16x32tile.tx",
                           "--tile 16 32", showinfo=True)

# Test --separate and --compression
command += maketx_command ("checker.tif", "checker-seplzw.tx",
                           "--separate --compression lzw", showinfo=True)

# Test --wrap
command += maketx_command ("checker.tif", "checker-clamp.tx",
                           "--wrap clamp", showinfo=True)

# Test --swrap and --twrap
command += maketx_command ("checker.tif", "checker-permir.tx",
                           "--swrap periodic --twrap mirror", showinfo=True)

# Test --nomipmap
command += maketx_command ("checker.tif", "checker-nomip.tx",
                           "--nomipmap", showinfo=True)

# Test --Mcamera, --Mscreen
command += maketx_command ("checker.tif", "checker-camera.tx",
                           "--Mcamera 1 0 0 0 0 2 0 0 0 0 1 0 0 0 0 1 --Mscreen 3 0 0 0 0 3 0 0 0 0 3 0 1 2 3 1",
                           showinfo=True)

# Test --opaque-detect (should drop the alpha channel)
command += maketx_command ("checker.tif", "checker-opaque.tx",
                           "--opaque-detect", showinfo=True)

# Test --monochrome-detect (first create a monochrome image)
command += oiiotool (" --pattern constant:color=.25,.25,.25 256x256 3 "
                    + " -d uint8 -o " + make_relpath("gray.tif"))
command += maketx_command ("gray.tif", "gray-mono.tx",
                           "--monochrome-detect", showinfo=True)

# Test --monochrome-detect on something that is NOT monochrome
command += oiiotool (" --pattern constant:color=.25,.2,.15 256x256 3 "
                    + " -d uint8 -o " + make_relpath("pink.tif"))
command += maketx_command ("pink.tif", "pink-mono.tx",
                           "--monochrome-detect", showinfo=True)

# Test --prman : should save 'separate' planarconfig, and funny 64x32 tiles
# since we are specifying 16 bits, and it should save as 'int16' even though
# we asked for unsigned.
command += maketx_command ("checker.tif", "checker-prman.tx",
                           "-d uint16 --prman", showinfo=True)

# Test --checknan : take advantage of the bad.exr images in 
# testsuite/oiiotool-fixnan.  (Use --nomipmap to cut down on stats output)
command += maketx_command (OIIO_TESTSUITE_ROOT+"/oiiotool-fixnan/src/bad.exr",
                           "nan.exr", "--checknan --nomipmap")

# Test --fixnan : take advantage of the bad.exr images in 
# testsuite/oiiotool-fixnan.  (Use --nomipmap to cut down on stats output)
command += maketx_command (OIIO_TESTSUITE_ROOT+"/oiiotool-fixnan/src/bad.exr",
                           "nan.exr", "--fixnan box3 --nomipmap",
                           showinfo=True, showinfo_extra="--stats")

# Test --format to force exr even though it can't be deduced from the name.
command += maketx_command ("checker.tif", "checker-exr.pdq",
                           "--format exr", showinfo=True)

# Test that we cleanly replace any existing SHA-1 hash and ConstantColor
# hint in the ImageDescription of the input file.
command += oiiotool (" --pattern constant:color=1,0,0 64x64 3 "
            + " --caption \"foo SHA-1=1234abcd ConstantColor=[0.0,0,-0.0] bar\""
            + " -d uint8 -o " + make_relpath("small.tif") )
command += info_command ("small.tif", safematch=1);
command += maketx_command ("small.tif", "small.tx",
                           "--oiio --constant-color-detect", showinfo=True)

# Test that the oiio:SHA-1 hash is stable, and that that changing filter and
# using -hicomp result in different images and different hashes.
command += maketx_command ("../common/grid.tif", "grid-lanczos3.tx",
                           extraargs = "-filter lanczos3")
command += maketx_command ("../common/grid.tif", "grid-lanczos3-hicomp.tx",
                           extraargs = "-filter lanczos3 -hicomp")
command += info_command ("grid.tx",
                         extraargs="--metamatch oiio:SHA-1")
command += info_command ("grid-lanczos3.tx",
                         extraargs="--metamatch oiio:SHA-1")
command += info_command ("grid-lanczos3-hicomp.tx",
                         extraargs="--metamatch oiio:SHA-1")

# Regression test -- at one point, we had a bug where we were botching
# the poles of OpenEXR env maps, adding energy.  Check it by creating an
# all-white image, turning it into an env map, and calculating its
# statistics (should be 1.0 everywhere).
command += oiiotool (" --pattern constant:color=1,1,1 4x2 3 "
            + " -d half -o " + make_relpath("white.exr"))
command += maketx_command ("white.exr", "whiteenv.exr",
                           "--envlatl")
command += oiiotool ("--stats -a whiteenv.exr")

# Test --lightprobe
command += maketx_command ("src/uffizi_probe-128.exr", "uffizi_latlong_env-128.exr",
                           "--lightprobe -d half")
outputs += [ "uffizi_latlong_env-128.exr" ]

# Test --bumpslopes to export a 6 channels height map with gradients
command += maketx_command ("noise.exr", "bumpslope.exr",
                           "--bumpslopes -d half", showinfo=True)

# Test --cdf
command += maketx_command ("noise.exr", "cdf.exr",
                           "--cdf -d half", showinfo=True)

command += maketx_command ("noise.exr", "bumpslope-cdf.exr",
                           "--bumpslopes --cdf -d half", showinfo=True)

# Test --sattrib, --attrib
command += maketx_command ("checker.tif", "checker-attribs.tx",
                           "--sattrib sname sval " +
                           "--attrib iname 42 " +
                           "--attrib fname 3.14159 " +
                           "--attrib sname2 sval2 ",
                           showinfo = True)

# Test --colorconvert and --unpremult
command += oiiotool (" --pattern constant:color=.25,.25,.25,.5 64x64 4 "
                    + " -d uint8 -o " + make_relpath("gray64srgb.tif"))
command += maketx_command ("gray64srgb.tif", "gray64linsrgb.tx",
                           "--colorconvert srgb lin_srgb --unpremult")

outputs += [ "out.txt" ]



# To do:  --fullpixels
#         --ignore-unassoc
#         --mipimage 
#         --fovcot



#!/usr/bin/env python

# Copyright Contributors to the OpenImageIO project.
# SPDX-License-Identifier: Apache-2.0
# https://github.com/AcademySoftwareFoundation/OpenImageIO

from __future__ import print_function
from __future__ import absolute_import
import OpenImageIO as oiio
import numpy as np


# Read the one subimage from input then write it to output using
# write_image, write_scanlines, write_scanline, write_tile, or write_tiles,
# depending on the 'method' argument).  (Just copy one subimage, one MIP
# level.)
def copy_subimage (input, output, method="image",
                   memformat=oiio.TypeFloat) :
    spec = input.spec ()
    if method == "image" :
        pixels = input.read_image (memformat)
        if pixels is None :
            print ("Error reading input pixels in", in_filename)
            return False
        output.write_image (pixels)
    elif method == "scanlines" and spec.tile_width == 0 :
        pixels = input.read_image (memformat)
        if pixels is None :
            print ("Error reading input pixels in", in_filename)
            return False
        output.write_scanlines (spec.y, spec.y+spec.height, spec.z,
                                pixels)
    elif method == "scanline" and spec.tile_width == 0 :
        for z in range(spec.z, spec.z+spec.depth) :
            for y in range(spec.y, spec.y+spec.height) :
                pixels = input.read_scanline (y, z, memformat)
                if pixels is None :
                    print ("Error reading input pixels in", in_filename)
                    return False
                output.write_scanline (y, z, pixels)
    elif method == "tiles" and spec.tile_width != 0 :
        pixels = input.read_image (memformat)
        if pixels is None :
            print ("Error reading input pixels in", in_filename)
            return False
        output.write_tiles (spec.x, spec.x+spec.width,
                            spec.y, spec.y+spec.height,
                            spec.z, spec.z+spec.depth,
                            pixels)
    elif method == "tile" and spec.tile_width != 0 :
        for z in range(spec.z, spec.z+spec.depth, spec.tile_depth) :
            for y in range(spec.y, spec.y+spec.height, spec.tile_height) :
                for x in range(spec.x, spec.x+spec.width, spec.tile_width) :
                    pixels = input.read_tile (x, y, z, memformat)
                    if pixels is None :
                        print ("Error reading input pixels in", in_filename)
                        return False
                    output.write_tile (x, y, z, pixels)
    else :
        print ("Unknown method:", method)
        return False
    return True


# Read the whole image then write using write_image, write_scanlines,
# write_scanline, write_tile, or write_tiles, depending on the 'method'
# argument).  (Just copy one subimage, one MIP level.)
def copy_image (in_filename, out_filename, method="image",
                memformat=oiio.TypeFloat, outformat=oiio.TypeUnknown) :
    input = oiio.ImageInput.open (in_filename)
    if not input :
        print ('Could not open "' + filename + '"')
        print ("\tError: ", oiio.geterror())
        print ()
        return
    outspec = input.spec()
    if outformat != oiio.TypeUnknown :
        outspec.format = outformat
    output = oiio.ImageOutput.create (out_filename)
    if not output :
        print ("Could not create ImageOutput for", out_filename)
        return
    ok = output.open (out_filename, outspec)
    if not ok :
        print ("Could not open", out_filename)
        return
    ok = copy_subimage (input, output, method, memformat)
    input.close ()
    output.close ()
    if ok :
        print ("Copied", in_filename, "to", out_filename, "as", method,
               "(memformat", memformat, "outformat", outformat, ")")


def test_subimages (out_filename="multipart.exr") :
    output = oiio.ImageOutput.create (out_filename)
    spec = oiio.ImageSpec (64, 64, 3, "half")
    specs = (spec, spec, spec)
    output.open (out_filename, specs)
    buffer = np.zeros ((64, 64, 3), dtype=float)
    for i in range(3) :
        if i != 0 :
            output.open (out_filename, specs[i], "AppendSubimage")
        output.write_image (buffer)
    output.close ()

######################################################################
# main test starts here

try:

    copy_image ("scanline.tif", "grid-image.tif", method="image")
    copy_image ("scanline.tif", "grid-scanline.tif", method="scanline")
    copy_image ("scanline.tif", "grid-scanlines.tif", method="scanlines")
    copy_image ("tiled.tif", "grid-timage.tif", method="image")
    copy_image ("tiled.tif", "grid-tile.tif", method="tile")
    copy_image ("tiled.tif", "grid-tiles.tif", method="tiles")

    # Regression test for crash when changing formats
    copy_image ("scanline.tif", "grid-image.tif",
                memformat=oiio.TypeUInt8, outformat=oiio.TypeUInt16)

    # Exercise 'half'
    copy_image ("scanline.tif", "grid-half.exr",
                memformat='half', outformat='half')

    # Ensure we can write multiple subimages
    test_subimages ()

    print ("Done.")
except Exception as detail:
    print ("Unknown exception:", detail)




#!/usr/bin/env python

# Copyright Contributors to the OpenImageIO project.
# SPDX-License-Identifier: Apache-2.0
# https://github.com/AcademySoftwareFoundation/OpenImageIO

from __future__ import print_function
from __future__ import absolute_import


############################################################################
# This file contains code examples from the ImageOutput chapter of the
# main OpenImageIO documentation.
#
# To add an additional test, replicate the section below. Change
# "example1" to a helpful short name that identifies the example.


# BEGIN-imageoutput-example1
import OpenImageIO as oiio
import numpy as np

def example1() :
    #
    # Example code fragment from the docs goes here.
    #
    # It probably should generate either some text output (which will show up
    # in "out.txt" that captures each test's output), or it should produce a
    # (small) image file that can be compared against a reference image that
    # goes in the ref/ subdirectory of this test.
    #
    return

# END-imageoutput-example1

#
############################################################################



# BEGIN-imageoutput-simple
import OpenImageIO as oiio
import numpy as np

def simple_write() :
    filename = "simple.tif"
    xres = 320
    yres = 240
    channels = 3  # RGB
    pixels = np.zeros((yres, xres, channels), dtype=np.uint8)

    out = oiio.ImageOutput.create (filename)
    if out:
        spec = oiio.ImageSpec(xres, yres, channels, 'uint8')
        out.open (filename, spec)
        out.write_image (pixels)
        out.close ()
# END-imageoutput-simple


import OpenImageIO as oiio
import numpy as np

def scanlines_write() :
    filename = "scanlines.tif"
    xres = 320
    yres = 240
    channels = 3  # RGB
    spec = oiio.ImageSpec(xres, yres, channels, 'uint8')

    out = oiio.ImageOutput.create (filename)
    if out:
        # BEGIN-imageoutput-scanlines
        z = 0   # Always zero for 2D images
        out.open (filename, spec)
        for y in range(yres) :
            # Generate pixel array for one scanline.
            # As an example, we are just making a zero-filled scanline
            scanline = np.zeros((xres, channels), dtype=np.uint8)
            out.write_scanline (y, z, scanline)
        out.close ()
        # END-imageoutput-scanlines



if __name__ == '__main__':
    # Each example function needs to get called here, or it won't execute
    # as part of the test.
    simple_write()
    scanlines_write()



#!/usr/bin/env python

import OpenImageIO as oiio

# Read the one subimage from input then write it to output using
# copy_image, thus skipping both decompression and recompression steps
# (Just copy one subimage, one MIP level.)
def copy_image (in_filename, out_filename) :
    input = oiio.ImageInput.open (in_filename)
    if not input :
        print ('Could not open "' + in_filename + '"')
        print ("\tError: ", oiio.geterror())
        print ()
        return
    outspec = input.spec()
    output = oiio.ImageOutput.create (out_filename)
    if not output :
        print ("Could not create ImageOutput for", out_filename)
        return
    ok = output.open (out_filename, outspec)
    if not ok :
        print ("Could not open", out_filename)
        return
    ok = output.copy_image(input)
    input.close ()
    output.close ()
    if ok :
        print ("Copied", in_filename, "to", out_filename)

# Copy lossy compressed image, this should not recompress the image again (loss of data).
copy_image("ref/compressed-pxr24.exr", "compressed-pxr24.exr")
copy_image("ref/compressed-b44.exr", "compressed-b44.exr")
copy_image("ref/compressed-b44a.exr", "compressed-b44a.exr")
copy_image("ref/compressed-dwaa.exr", "compressed-dwaa.exr")
copy_image("ref/compressed-dwab.exr", "compressed-dwab.exr")


#!/usr/bin/env python

# Copyright Contributors to the OpenImageIO project.
# SPDX-License-Identifier: Apache-2.0
# https://github.com/AcademySoftwareFoundation/OpenImageIO


# This test just maps a 50% grey texture, once with default settings, and once
# declaring that it thinks the texture is in sRGB texture space.

# Note: we deliberately make the two output images different sizes so that
# they can't match against each others' ref images.

command += oiiotool ("-pattern constant:color=0.5,0.5,0.5 64x64 3 -d uint8 -otex grey.exr")
command += testtex_command ("-res 64 64 --no-gettextureinfo --nowarp grey.exr -o nocc.exr")
command += testtex_command ("-res 60 60 --no-gettextureinfo --nowarp --texcolorspace sRGB grey.exr -o cc.exr")
outputs = [ "nocc.exr", "cc.exr", "out.txt" ]



#!/usr/bin/env python

# Copyright Contributors to the OpenImageIO project.
# SPDX-License-Identifier: Apache-2.0
# https://github.com/AcademySoftwareFoundation/OpenImageIO


command = testtex_command ("../common/textures/grid.tx",
                           extraargs = "-interpmode 1  -d uint8 -o out.tif")
outputs = [ "out.tif" ]



#!/usr/bin/env python

# Copyright Contributors to the OpenImageIO project.
# SPDX-License-Identifier: Apache-2.0
# https://github.com/AcademySoftwareFoundation/OpenImageIO

imagedir = "ref/"
files = [ "IMG_7702_small.heic", "Chimera-AV1-8bit-162.avif" ]
for f in files:
    command = command + info_command (os.path.join(imagedir, f))

files = [ "greyhounds-looking-for-a-table.heic", "sewing-threads.heic" ]
for f in files:
    command = command + info_command (os.path.join(OIIO_TESTSUITE_IMAGEDIR, f))

# avif conversion is expected to fail if libheif is built without AV1 support
failureok = 1



#!/usr/bin/env python

# Copyright Contributors to the OpenImageIO project.
# SPDX-License-Identifier: Apache-2.0
# https://github.com/AcademySoftwareFoundation/OpenImageIO


from __future__ import absolute_import
import os

files = [ "RAW_CANON_EOS_7D.CR2",
          "RAW_NIKON_D3X.NEF",
          "RAW_FUJI_F700.RAF",
          "RAW_NIKON_D3X.NEF",
          "RAW_OLYMPUS_E3.ORF",
          "RAW_PANASONIC_G1.RW2",
          "RAW_PENTAX_K200D.PEF",
          "RAW_SONY_A300.ARW" ]
outputs = []

# Things vary a lot with libraw versions.
# FIXME -- return to this later
if (os.getenv('GITHUB_ACTIONS') == 'true'):
    failthresh = 0.024
    files.remove ("RAW_PANASONIC_G1.RW2")

# Fairly high hard fail, since libraw seems to diddle with its debayering
# from version to version, it's hard to make a single reference image.
hardfail = 0.017

# For each test image, read it and print all metadata, resize it (to make
# the ref images small) and compared to the reference.
for f in files:
    outputname = f+".tif"
    command += oiiotool ("-iconfig raw:ColorSpace linear "
                         + "-i:info=2 " + OIIO_TESTSUITE_IMAGEDIR + "/" + f
                         + " -resample '5%' -d uint8 "
                         + "-o " + outputname)
    outputs += [ outputname ]

outputs += [ "out.txt" ]



#!/usr/bin/env python

# Copyright Contributors to the OpenImageIO project.
# SPDX-License-Identifier: Apache-2.0
# https://github.com/AcademySoftwareFoundation/OpenImageIO

# ../fits-image/pg93:
# tst0001.fits to tst0014.fits
imagedir = OIIO_TESTSUITE_IMAGEDIR + "/pg93"
files = [ "tst0001.fits", "tst0003.fits",
           "tst0005.fits", "tst0006.fits", "tst0007.fits",  "tst0008.fits",
           #FIXME? "tst0009.fits",
          "tst0013.fits"
        ]
for f in files :
    command += rw_command (imagedir, f)

imagedir = OIIO_TESTSUITE_IMAGEDIR + "/ftt4b"
files = [ "file001.fits", "file002.fits", "file003.fits",
          "file009.fits", "file012.fits" ]
for f in files :
    command += rw_command (imagedir, f)



#!/usr/bin/env python

# Copyright Contributors to the OpenImageIO project.
# SPDX-License-Identifier: Apache-2.0
# https://github.com/AcademySoftwareFoundation/OpenImageIO


command += maketx_command  ("../common/textures/grid.tx",
                            "grid-uint16.tx",
                            "-d uint16", showinfo=True)
command += testtex_command ("grid-uint16.tx")
outputs = [ "out.exr" ]
